/*
 * sumo.c
 */

#include <stdlib.h>
#include <string.h>
#include "sumo.h"
#include "dither.h"

#ifdef BIG_ENDIAN
#define RED_SHIFT32		0
#define GREEN_SHIFT32	8
#define BLUE_SHIFT32	16
#else
#define RED_SHIFT32		16
#define GREEN_SHIFT32	8
#define BLUE_SHIFT32	0
#endif

#define RED_MASK32		(0xff << RED_SHIFT32)
#define GREEN_MASK32	(0xff << GREEN_SHIFT32)
#define BLUE_MASK32		(0xff << BLUE_SHIFT32)

#define PACK_RGB16(r, g, b) \
	(((r & 0x1f) << 11) | ((g & 0x3f) << 5) | (b & 0x1f))

#define FIXED_BITS 16

static int scaling = 1;
static int dither = 1;
static int xsz, ysz;
static int dst_xsz, dst_ysz, dst_bpp;
static int inv_dst_xsz, inv_dst_ysz;
static int crop_x, crop_y;
static unsigned int bg_color = 0xFF000000;

static unsigned int get_src_pixel(int x, int y, unsigned int *pixels);

void sumo_init(int width, int height)
{
	sumo_scr_detect();
	xsz = width;
	ysz = height;
	dst_xsz = sumo_scr_width();
	dst_ysz = sumo_scr_height();
	dst_bpp = sumo_scr_bpp();
	inv_dst_xsz = (1 << FIXED_BITS) / dst_xsz;
	inv_dst_ysz = (1 << FIXED_BITS) / dst_ysz;
	crop_x = (dst_xsz - xsz) / 2;
	crop_y = (dst_ysz - ysz) / 2;
}

void sumo_set_scaling(int enable)
{
	scaling = enable;
}

void sumo_set_dither(int enable)
{
	dither = enable;
}

void sumo_update(void *pixels)
{
	unsigned char *dest = sumo_scr_pixels();
	unsigned int p, red, green, blue;

	if(dst_bpp == 32 && dst_xsz == xsz && dst_ysz == ysz) {
		memcpy(dest, pixels, xsz * ysz * 4);
	} else {
		int i, j;
		for(j=0; j<dst_ysz; j++) {
			for(i=0; i<dst_xsz; i++) {
				p = get_src_pixel(i, j, pixels);
				switch(dst_bpp) {
				case 32:
					*(unsigned int*)dest = p;
					dest += 4;
					break;

				case 24:
					red = (p & RED_MASK32) >> RED_SHIFT32;
					green = (p & GREEN_MASK32) >> GREEN_SHIFT32;
					blue = (p & BLUE_MASK32) >> BLUE_SHIFT32;
					*dest++ = blue;
					*dest++ = green;
					*dest++ = red;
					break;

				case 16:
					red = (p & RED_MASK32) >> RED_SHIFT32;
					green = (p & GREEN_MASK32) >> GREEN_SHIFT32;
					blue = (p & BLUE_MASK32) >> BLUE_SHIFT32;
					*(unsigned short*)dest = PACK_RGB16(red >> 3, green >> 2, blue >> 3);
					dest += 2;
					break;

				case 8:
					if (dither)
					{
						*dest++ = dither332(p, i, j);
					}
					else
					{
						red = (p & RED_MASK32) >> RED_SHIFT32;
						green = (p & GREEN_MASK32) >> GREEN_SHIFT32;
						blue = (p & BLUE_MASK32) >> BLUE_SHIFT32;
						*dest++ = (red & 0xE0) | ((green >> 2) & 0x0C) | (blue >> 6);
					}
					break;
				default:
					break;
				}
			}
		}
	}
}

char *sumo_get_text(void)
{
	return 0;	/* TODO */
}

enum sumo_text_type sumo_get_text_type(void)
{
	return SUMO_TEXT_MSG;	/* TODO */
}

void *sumo_get_text_image(void)
{
	return 0;	/* TODO */
}

static unsigned int get_src_pixel(int x, int y, unsigned int *pixels)
{
	static int nx, ny;
	if (xsz == dst_xsz && ysz == dst_ysz)
	{
		return pixels[x + xsz * y];
	}
	if (scaling)
	{
		nx = ((x * xsz) * inv_dst_xsz) >> FIXED_BITS;
		ny = ((y * ysz) * inv_dst_ysz) >> FIXED_BITS;
	}
	else
	{
		nx = x - crop_x;
		ny = y - crop_y;
	}

	if (nx < 0 || nx >= xsz || ny < 0 || ny >= ysz)
		return bg_color;
	
	return pixels[nx + xsz * ny];
}

