#include <stdio.h>
#include "dither.h"
#include <SDL.h>
#include "sumo.h"

static SDL_Surface *fbsurf;

int _sumo_emul_w = 640;
int _sumo_emul_h = 480;
int _sumo_emul_bpp = 8;
int _sumo_emul_fs = 0;

static int start_ticks;

void sumo_scr_detect()
{
	static int first_time = 1;
	int w, h, bpp;
	char fs;
	FILE *fp;
	
	/* just in case someone calls this function twice
	 * while the emulator is running
	 */
	if (!first_time) return;
	first_time = 0;

	fp = fopen("sumo_emu_config", "rb");
	if (fp)
	{
		fscanf(fp, "%dx%dx%d%c", &w, &h, &bpp, &fs);
		fclose(fp);
		_sumo_emul_w = w;
		_sumo_emul_h = h;
		_sumo_emul_bpp = bpp;
		_sumo_emul_fs = fs == 'f' ? 1 : 0;
	}

	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
	if(!(fbsurf = SDL_SetVideoMode(_sumo_emul_w, _sumo_emul_h, _sumo_emul_bpp, SDL_SWSURFACE | (_sumo_emul_fs ? SDL_FULLSCREEN : 0)))) {
		exit(0);
	}
	SDL_WM_SetCaption("sumo emulator v0.1", 0);
	SDL_ShowCursor(0);
	if (_sumo_emul_bpp == 8)
	{
		/* setup palette */
		SDL_Color colors[256];
		int i;
		for (i=0; i<256; i++)
		{
			colors[i].r = r332(i);
			colors[i].g = g332(i);
			colors[i].b = b332(i);
		}
		SDL_SetPalette(fbsurf, SDL_LOGPAL|SDL_PHYSPAL, colors, 0, 256);
	}

	atexit(SDL_Quit);
	start_ticks = SDL_GetTicks();
}

int sumo_scr_width()
{
	return _sumo_emul_w;
}

int sumo_scr_height()
{
	return _sumo_emul_h;
}

int sumo_scr_bpp()
{
	return _sumo_emul_bpp;
}

char *sumo_scr_pixels()
{
	int ticks;
	static int frames = 0;
	static int first_time = 1;
	if (!first_time)
	{
		SDL_Event event;
		if (SDL_MUSTLOCK(fbsurf)) SDL_UnlockSurface(fbsurf);
		SDL_Flip(fbsurf);
		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_ESCAPE) {
					exit(0);
				}

			default:
				break;
			}
		}
	}
	first_time = 0;

	if (SDL_MUSTLOCK(fbsurf)) SDL_LockSurface(fbsurf);
	
	frames++;
	ticks = sumo_ticks();
	if (ticks)
	{
		printf("\r%d frames in %d ms: %f fps", frames, ticks, 
			(float) frames / (float) (ticks / 1000.0f));
	}

	return fbsurf->pixels;
}

unsigned int sumo_ticks()
{
	return SDL_GetTicks() - start_ticks;
}